/*
 *
 *  Angepasst an meine BLUEPOWER-Steuerung:
 *   RTS-Impuls (LHL) schaltet das Netzrelais mit Verzgerung von etwa drei
 *   Sekunden ab, CTS kommt von den Schaltern (high=an).
 *
 */
 
/*  backupsd.c -- Simple Daemon to catch power failure signals from a
 *                Back-UPS (from APC).
 *
 *
 *  Parts of the code are from Miquel van Smoorenburg's powerd.c
 *  Other parts are original from Christian Holtje <docwhat@uiuc.edu>
 *  I believe that it is okay to say that this is Public Domain, just
 *  give credit, where credit is due.
 *
 *  Disclaimer:  We make NO claims to this software, and take no
 *               resposibility for it's use/misuse.
 */

#include <sys/types.h>
#include <sys/ioctl.h>
#include <fcntl.h>
#include <errno.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <signal.h>
#include <syslog.h>

#define ever (;;)

/* This is the file needed by SysVInit */
#define PWRSTAT         "/etc/powerstatus"
#define PULSELENGTH     500000

/* Tell init the power has either gone or is back. */
void powerfail (int ok)
{
	int fd;
	
	syslog(LOG_DEBUG,"powerfail ok=%i",ok);

	/* Create an info file needed by init to shutdown/cancel shutdown */
	unlink (PWRSTAT);    
	if ((fd=open (PWRSTAT,O_CREAT|O_WRONLY,0644))>=0) {
		if (ok) {
			write (fd,"OK\n",3);
		} else {
			write (fd,"FAIL\n",5);
		}
		close(fd);
	}
	kill (1,SIGPWR);
}


/* Main program. */
int main(int argc, char **argv) 
{
	int fd;
	int PowerBit = TIOCM_RTS;
	int flags;
	int status;
	int oldstat = -1;
	int count = 0;
	int i;
	pid_t pid;

	/* Open the the device */
	if ((fd=open ("/dev/bluepower",O_RDWR | O_NDELAY)) < 0) {
		perror ("BluePower");
		return (1);
	}

	if ((argc>=2)  && (strcmp(argv[1], "killpower")==0) ) {
		/* Let's kill the power! */
		printf ("Attempting to switch off power.\nPulsing RTS ... "); 
		fflush (stdout);
		ioctl (fd,TIOCMBIC,&PowerBit); /* Clear RTS */
		printf ("low ... ");
		fflush (stdout);
		usleep (PULSELENGTH);
		ioctl (fd,TIOCMBIS,&PowerBit); /* Set RTS */
		printf ("high ... ");
		fflush (stdout);
		usleep (PULSELENGTH);
		ioctl (fd,TIOCMBIC,&PowerBit); /* Clear RTS */
		/* jetzt dauert es noch etwa drei Sekunden ... */
		printf ("low.\nPower should be switched off in three seconds.\n");
		fflush (stdout);
		return (0);
	}

	/* clear outputs RTS and DTR */
    
	i=TIOCM_RTS + TIOCM_DTR;
	ioctl (fd,TIOCMBIC,&i);
 	fflush (stdout);

	/* Become a daemon. */
	switch (pid=fork()) {
	case 0  : /* I am the child. */
		break;
    case -1 : /* Failed to become daemon. */
		perror ("BluePower: can't fork");
		return (1);
    default : /* I am the parent. */
    	printf ("BluePower: daemon PID is %i\n",pid);
		return (0);
	}
	
	openlog("BluePower",LOG_PID,LOG_DAEMON);
  
	printf ("BluePower: daemon loaded.\n");
	syslog(LOG_DEBUG,"daemon loaded.");
	setsid();
  
	/* Now sample the CTS line. */
	for ever {
		ioctl (fd,TIOCMGET,&flags);
		status = (flags & TIOCM_CTS) ? 0 : 1; 
		/* Did CTS jumps to low? Then the power has failed. */
		if (oldstat==0 && status!=0) {
			syslog(LOG_DEBUG,"Detected falling edge of control line");
			count++;
			if (count > 3) {
				powerfail (0);
			} else {
				sleep (1); 
				continue; 
			}
		}
		/* Did CTS go up again? Then the power is back. */
		if (oldstat > 0 && status == 0) {
			syslog(LOG_DEBUG,"Detected raising edge of control line");
			count++;
			if (count > 3) {
				powerfail (1);
			} else { 
				sleep (1); 
				continue; 
			}
		}
		/* Reset count, remember status and sleep 2 seconds. */
		count = 0;
		oldstat = status;
		sleep (2);
	}
	/* Error! (shouldn't happen) */
	printf ("BluePower: This should not happen: infinite loop ended !!!");
	return(1);
}
